// **********************************************************************
//
// Copyright (c) 2003-2017 ZeroC, Inc. All rights reserved.
//
// This copy of Ice is licensed to you under the terms described in the
// ICE_LICENSE file included in this distribution.
//
// **********************************************************************
//
// Ice version 3.7.0
//
// <auto-generated>
//
// Generated from file `ValueFactory.ice'
//
// Warning: do not edit this file.
//
// </auto-generated>
//

#ifndef __Ice_ValueFactory_h__
#define __Ice_ValueFactory_h__

#include <IceUtil/PushDisableWarnings.h>
#include <Ice/ProxyF.h>
#include <Ice/ObjectF.h>
#include <Ice/ValueF.h>
#include <Ice/Exception.h>
#include <Ice/LocalObject.h>
#include <Ice/StreamHelpers.h>
#include <Ice/Comparable.h>
#include <IceUtil/ScopedArray.h>
#include <Ice/Optional.h>
#include <IceUtil/UndefSysMacros.h>

#ifndef ICE_IGNORE_VERSION
#   if ICE_INT_VERSION / 100 != 307
#       error Ice version mismatch!
#   endif
#   if ICE_INT_VERSION % 100 > 50
#       error Beta header file detected
#   endif
#   if ICE_INT_VERSION % 100 < 0
#       error Ice patch level mismatch!
#   endif
#endif

#ifndef ICE_API
#   if defined(ICE_STATIC_LIBS)
#       define ICE_API /**/
#   elif defined(ICE_API_EXPORTS)
#       define ICE_API ICE_DECLSPEC_EXPORT
#   else
#       define ICE_API ICE_DECLSPEC_IMPORT
#   endif
#endif

#ifdef ICE_CPP11_MAPPING // C++11 mapping

namespace Ice
{

class ValueFactoryManager;

}

namespace Ice
{

using ValueFactory = ::std::function<::std::shared_ptr<::Ice::Value>(const ::std::string&)>;

class ICE_CLASS(ICE_API) ValueFactoryManager
{
public:

    ICE_MEMBER(ICE_API) virtual ~ValueFactoryManager();

    virtual void add(::Ice::ValueFactory, const ::std::string&) = 0;

    virtual ::Ice::ValueFactory find(const ::std::string&) const = 0;
};

}

namespace Ice
{

}

namespace Ice
{

using ValueFactoryManagerPtr = ::std::shared_ptr<ValueFactoryManager>;

}

#else // C++98 mapping

namespace Ice
{

class ValueFactory;
ICE_API ::Ice::LocalObject* upCast(::Ice::ValueFactory*);
typedef ::IceInternal::Handle< ::Ice::ValueFactory> ValueFactoryPtr;

class ValueFactoryManager;
ICE_API ::Ice::LocalObject* upCast(::Ice::ValueFactoryManager*);
typedef ::IceInternal::Handle< ::Ice::ValueFactoryManager> ValueFactoryManagerPtr;

}

namespace Ice
{

class ICE_API ValueFactory : public virtual ::Ice::LocalObject
{
public:

    typedef ValueFactoryPtr PointerType;

    virtual ~ValueFactory();

    virtual ::Ice::ValuePtr create(const ::std::string&) = 0;
};

inline bool operator==(const ValueFactory& lhs, const ValueFactory& rhs)
{
    return static_cast<const ::Ice::LocalObject&>(lhs) == static_cast<const ::Ice::LocalObject&>(rhs);
}

inline bool operator<(const ValueFactory& lhs, const ValueFactory& rhs)
{
    return static_cast<const ::Ice::LocalObject&>(lhs) < static_cast<const ::Ice::LocalObject&>(rhs);
}

class ICE_API ValueFactoryManager : public virtual ::Ice::LocalObject
{
public:

    typedef ValueFactoryManagerPtr PointerType;

    virtual ~ValueFactoryManager();

    virtual void add(const ::Ice::ValueFactoryPtr&, const ::std::string&) = 0;

    virtual ::Ice::ValueFactoryPtr find(const ::std::string&) const = 0;
};

inline bool operator==(const ValueFactoryManager& lhs, const ValueFactoryManager& rhs)
{
    return static_cast<const ::Ice::LocalObject&>(lhs) == static_cast<const ::Ice::LocalObject&>(rhs);
}

inline bool operator<(const ValueFactoryManager& lhs, const ValueFactoryManager& rhs)
{
    return static_cast<const ::Ice::LocalObject&>(lhs) < static_cast<const ::Ice::LocalObject&>(rhs);
}

}

namespace Ice
{

}

#endif

#include <IceUtil/PopDisableWarnings.h>
#endif
