// **********************************************************************
//
// Copyright (c) 2003-2017 ZeroC, Inc. All rights reserved.
//
// This copy of Ice is licensed to you under the terms described in the
// ICE_LICENSE file included in this distribution.
//
// **********************************************************************

#ifndef ICE_VALUE_F_H
#define ICE_VALUE_F_H

#ifdef ICE_CPP11_MAPPING
#include <Ice/Config.h>

namespace Ice
{

class Value;
using ValuePtr = ::std::shared_ptr<Value>;

}
#endif
#endif
